<?php
class Mage_Opay_Model_PaymentMethod extends Mage_Payment_Model_Method_Abstract
{
    const APP_NAME = 'Magento';
    const WIDGET_VERSION = '1.3.1';

    protected $_isGateway = true;
    protected $_canAuthorize = true;
    protected $_canCapture = true;

    protected $_code = 'opay';
    protected $_formBlockType = 'opay/form';

    public function getTitle()
    {
        $paymentGatewayName = $this->getSettings('payment_gateway_name');

        if (!empty($paymentGatewayName))
        {
            return $paymentGatewayName;
        }
        else
        {
            return Mage::helper('opay')->__('OPAY Payment Gateway');
        }
    }

    public function assignData($data)
    {
        if (!($data instanceof Varien_Object)) {
            $data = new Varien_Object($data);
        }
        $info = $this->getInfoInstance();
        $info->setAdditionalInformation('banklink_option', $data->getBanklinkOption());

        return $this;
    }

    public function validate()
    {
        parent::validate();

        $info = $this->getInfoInstance();

        $banklink = $info->getBanklinkOption();

        return $this;
    }

    public function validateBanklinkOption()
    {
        $return = "<script type=\"text/javascript\">
        //<![CDATA[
            function whenObjIsAvailable(name, callback) {
                var interval = 10,
                    callCount = 0;
                setTimeout(function() {
                    if (window[name]) {
                        callback(window[name]);
                    } else {
                        if (callCount < 1500) {
                            callCount++;
                            setTimeout(arguments.callee, interval);
                        }
                    }
                }, interval);
            }

            whenObjIsAvailable('payment', function() {

                var isChannelSelected = function() {
                    var radList = document.getElementsByName('payment[banklink_option]');
                    for (var i = 0, l = radList.length; i < l; i++)
                    {
                        if(radList[i].checked)
                        {
                            return true;
                        }
                    }
                    return false;
                };

                Payment.prototype.save = Payment.prototype.save.wrap(function(save) {
                    if (document.getElementById('payment_form_opay').style.display != 'none')
                    {
                        if (!isChannelSelected())
                        {
                            alert('".Mage::helper('opay')->__('Please specify payment method.')."');
                        }
                        else
                        {
                            save();
                        }
                    }
                    else
                    {
                        save();
                    }
                });

                if (typeof IWD != 'undefined' && typeof IWD.OPC != 'undefined') {
                    IWD.OPC.saveOrder = IWD.OPC.saveOrder.wrap(function(save) {
                        if (document.getElementById('payment_form_opay').style.display != 'none')
                        {
                            if (!isChannelSelected())
                            {
                                var msg = '".Mage::helper('opay')->__('Please specify payment method.')."';
                                if (jQuery('.opc-message-container').length
                                    && jQuery('.opc-message-wrapper').length
                                ) {
                                    jQuery('.opc-message-container').html(msg);
                                    jQuery('.opc-message-wrapper').show();
                                } else {
                                    alert(msg);
                                }
                                IWD.OPC.saveOrderStatus = false;
                                IWD.OPC.Checkout.hideLoader();
                                IWD.OPC.Checkout.unlockPlaceOrder();
                                return false;
                            }
                            else
                            {
                                return save();
                            }
                        }
                        else
                        {
                            return save();
                        }
                    });
                }
            });
        //]]>
        </script>";
        return $return;
    }

    public function getOrderPlaceRedirectUrl()
    {
        $time = $this->getTimestamp();
        return Mage::getUrl('opay/pay/redirect', array('timestamp' => $time));
    }

    private function getTimestamp()
    {
        if (function_exists('microtime')) {
            $time = round(microtime(true) * 1000);
        }
        if (empty($time)) {
            $time = time();
        }
        return $time;
    }

    public function getGateway()
    {
        require_once(Mage::getBaseDir() . '/app/code/local/Mage/Opay/OpayLibrary/opay_8.1.gateway.inc.php');
        $opay = new OpayGateway();

        $_code=$this->getMethodCode();

        $_pData    = Mage::getStoreConfig('payment/opay');

        $_customer = Mage::getSingleton('customer/session')->getCustomer();

        $order_id = Mage::getSingleton('checkout/session')->getQuoteId();

        $_totals = Mage::getSingleton('checkout/cart')->getQuote()->collectTotals()->getGrandTotal();

        $_currency = Mage::app()->getStore()->getCurrentCurrencyCode();

        $_customerIds = Mage::getStoreConfig('payment/opay');

        $_language = strtoupper(substr(Mage::app()->getLocale()->getLocaleCode(), 0, 2));

        if ($_customerIds['testmode'] == 0){
            $test = null;
        } elseif ($_customerIds['testmode'] == 1) {
            $test = $_customerIds['test'];
        }

        $checkout = Mage::getSingleton('checkout/session')->getQuote();
        $billAddress = $checkout->getBillingAddress()->getData();

        if($_pData['signature'] == 1) {

            $opay->setSignaturePassword($_customerIds['api_secret']);

        } elseif($_pData['signature'] == 0) {

            $opay->setMerchantRsaPrivateKey($_customerIds['api_rsa']);
            $opay->setOpayCertificate($_customerIds['api_rsa_opay']);
        }

        $paramsArray = array(

            'website_id'            => $_customerIds['api_key'],
            'order_nr'              => $order_id,
            'redirect_url'          => Mage::getUrl('opay/pay/userredirect'),
            'web_service_url'       => Mage::getUrl('opay/pay/callback'),
            'standard'              => 'opay_8.1',
            'country'               => $billAddress['country_id'],
            'language'              => $_language,
            'amount'                => intval(number_format(($_totals * 100), 0, '', '')),
            'currency'              => $_currency
        );

        if (!empty($test))
        {
            $paramsArray['test'] = $test;
        }

        try
        {
            $paramsArray = $opay->signArrayOfParameters($paramsArray);
            $array = $opay->webServiceRequest('https://gateway.opay.lt/api/listchannels/', $paramsArray);
        }
        catch (OpayGatewayException $e)
        {
            $array = array();
        }
        return $array;
    }


    public function getSettings($name = '')
    {
        if (empty($this->getSettingsArray))
        {

            require_once(Mage::getBaseDir() . '/app/code/local/Mage/Opay/OpayLibrary/opay_8.1.gateway.inc.php');
            $opay = new OpayGateway();

            $_pData    = Mage::getStoreConfig('payment/opay');
            $_customerIds = Mage::getStoreConfig('payment/opay');
            $_language = strtoupper(substr(Mage::app()->getLocale()->getLocaleCode(), 0, 2));

            $checkout = Mage::getSingleton('checkout/session')->getQuote();
            $billAddress = $checkout->getBillingAddress()->getData();

            if($_pData['signature'] == 1)
            {
                $opay->setSignaturePassword($_customerIds['api_secret']);
            }
            elseif($_pData['signature'] == 0)
            {
                $opay->setMerchantRsaPrivateKey($_customerIds['api_rsa']);
                $opay->setOpayCertificate($_customerIds['api_rsa_opay']);
            }

            $paramsArray = array(
                'service_name' => 'getSettings',
                'website_id'   => $_customerIds['api_key'],
                'language'     => $_language,
            );

            if (isset($billAddress['country_id'])) {
                $paramsArray['country'] = $billAddress['country_id'];
            }

            try
            {
                $paramsArray = $opay->signArrayOfParameters($paramsArray);
                $array = $opay->webServiceRequest('https://gateway.opay.lt/api/websites/', $paramsArray);
            }
            catch (OpayGatewayException $e)
            {
                $array = array();
            }

            if (!empty($array['response']['result']))
            {
                $this->getSettingsArray = $array['response']['result'];
            }
        }


        if ($name != '')
        {
            return (isset($this->getSettingsArray[$name])) ? $this->getSettingsArray[$name] : '';
        }
        else
        {
            return (!empty($this->getSettingsArray)) ? $this->getSettingsArray : array();
        }
    }

    public function getAutoSubmitForm()
    {
        require_once(Mage::getBaseDir() . '/app/code/local/Mage/Opay/OpayLibrary/opay_8.1.gateway.inc.php');
        $opay = new OpayGateway();

        $orderIncrementId = Mage::getSingleton('checkout/session')->getLastRealOrderId();
        $order = Mage::getModel('sales/order')->loadByIncrementId($orderIncrementId);

        $customerIds = Mage::getStoreConfig('payment/opay');

        if ($customerIds['testmode'] == 0){
            $test = null;
        } elseif ($customerIds['testmode'] == 1) {
            $test = $customerIds['test'];
        }

        //load order by order id
        $billing_address = $order->getBillingAddress();
        $billAd = $billing_address->getData();
        $phone = $billing_address->getTelephone();

        $language = strtoupper(substr(Mage::app()->getLocale()->getLocaleCode(), 0, 2));

        // Salies kodas
        $checkout = Mage::getSingleton('checkout/session')->getQuote();
        $billAddress = $checkout->getBillingAddress()->getData();

        $payment = $order->getPayment();
        $banklink_option = $payment->getAdditionalInformation('banklink_option');
        if($banklink_option == null){
            $banklink = null;
        } else {
            $banklink = $banklink_option;
        }

        if($customerIds['signature'] == 1) {

            $opay->setSignaturePassword($customerIds['api_secret']);
        } elseif($customerIds['signature'] == 0) {

            $opay->setMerchantRsaPrivateKey($customerIds['api_rsa']);

            $opay->setOpayCertificate($customerIds['api_rsa_opay']);

        }

        $paramsArray = array(

            'website_id'            => $customerIds['api_key'],
            'order_nr'              => $order->increment_id,
            'redirect_url'          => Mage::getUrl('opay/pay/userredirect'),
            'web_service_url'       => Mage::getUrl('opay/pay/callback'),
            'standard'              => 'opay_8.1',
            'country'               => $billAd['country_id'],
            'language'              => $language,
            'amount'                => intval(number_format(($order->grand_total * 100), 0, '', '')),
            'currency'              => $order->order_currency_code,
            'c_email'               => $order->customer_email,
            'c_mobile_nr'           => $phone,
        );

        if (!empty($test))
        {
            $paramsArray['test'] = $test;
        }

        if (!empty($banklink))
        {
            $paramsArray['pass_through_channel_name'] = $banklink;
        }


        $banklink_option = $banklink;
        if($banklink_option == null){
            $banklink_msg = null;
        } else {
            $banklink_msg = $banklink_option;
        }


        if(!empty($banklink_msg)){
            $comment = Mage::helper('opay')->__('Customer chose payment method') . ': ' . $banklink_msg . '.';
        } else{
            $comment = Mage::helper('opay')->__('Customer redirected to external OPAY page.');
        }

        $order->addStatusHistoryComment($comment);
        $order->save();

        $this->appendMetadataToParams($paramsArray);
        $paramsArray = $opay->signArrayOfParameters($paramsArray);

        $url = sprintf(
            'https://gateway.opay.lt/pay/?encoded=%s',
            $opay->convertArrayOfParametersToEncodedString($paramsArray)
        );

        header(sprintf('Location: %s', $url));
    }


    public function validateCallback()
    {
        require_once(Mage::getBaseDir() . '/app/code/local/Mage/Opay/OpayLibrary/opay_8.1.gateway.inc.php');
        $opay = new OpayGateway();
        $customerIds = Mage::getStoreConfig('payment/opay');

        if($customerIds['signature'] == 1) {

            $opay->setSignaturePassword($customerIds['api_secret']);

        } elseif($customerIds['signature'] == 0) {

            $opay->setMerchantRsaPrivateKey($customerIds['api_rsa']);

            $opay->setOpayCertificate($customerIds['api_rsa_opay']);
        }

        if (isset($_POST['encoded']))
        {
            $parametersArray = $opay->convertEncodedStringToArrayOfParameters($_POST['encoded']);
        }
        else if (isset($_GET['encoded']))
        {
            $parametersArray = $opay->convertEncodedStringToArrayOfParameters($_GET['encoded']);
        }
        else if (isset($_POST['password_signature']) || isset($_POST['rsa_signature']))
        {
            $parametersArray = $_POST;
        }
        else if (isset($_GET['password_signature']) || isset($_GET['rsa_signature']))
        {
            $parametersArray = $_GET;
        }


        if (isset($parametersArray))
        {
            if ($opay->verifySignature($parametersArray))
            {
                $order = Mage::getModel('sales/order')->loadByIncrementId($parametersArray['order_nr']);
                if ($order->increment_id)
                {
                    if ($parametersArray['status'] == 1)
                    {
                        if ($parametersArray['p_currency'] != $order->order_currency_code)
                        {
                            return 'Wrong currency: ' . $parametersArray['p_currency'];
                        }

                        $receive_amount = intval($parametersArray['p_amount']);
                        $local_amount   = intval(number_format(($order->grand_total * 100), 0, '', ''));

                        if ($receive_amount < $local_amount)
                        {
                            return 'Amount ' . $receive_amount . ' is wrong.';
                        }

                        try {

                            if (in_array($order->getState(), array(
                                Mage_Sales_Model_Order::STATE_NEW,
                                Mage_Sales_Model_Order::STATE_PENDING_PAYMENT
                            ))) {

                                // Locking order until order state is changed to avoid possible double payment captures when user redirect and callback are happening at the exact same time
                                $connection = Mage::getSingleton('core/resource')->getConnection('core_read');
                                $connection->beginTransaction();
                                $query = 'SELECT * FROM ' . Mage::getSingleton('core/resource')->getTableName('sales/order') . ' WHERE increment_id = \'' . $parametersArray['order_nr'] . '\' FOR UPDATE';
                                $results = $connection->fetchAll($query);

                                // Capturing payment
                                if ($order->canInvoice()) {
                                    $invoice = $order->prepareInvoice();
                                    $invoice->register()->capture();
                                    $order->addRelatedObject($invoice);
                                }

                                // Sending e-mail notification to customer and adding comment to history about that
                                if ($invoice && !$order->getEmailSent()) {

                                    if (method_exists($order, 'queueNewOrderEmail')) {
                                        $order->queueNewOrderEmail();
                                    } else {
                                        $order->sendNewOrderEmail();
                                    }

                                    $comment = Mage::helper('opay')->__('Customer notified by E-mail.');
                                    $history = $order->addStatusHistoryComment($comment);
                                    $history->setIsCustomerNotified(true);
                                }

                                $order->save();
                                $connection->commit();


                                Mage::getSingleton('checkout/session') -> getQuote() -> setIsActive(false) -> save();
                            }

                            exit('OK');
                        } catch (Exception $e) {
                            $connection->rollback();
                            return get_class($e) . ': ' . $e->getMessage();
                        }

                    }
                    elseif ($parametersArray['status'] == 0)
                    {
                        try
                        {
                            if (in_array($order->getState(), array(
                                Mage_Sales_Model_Order::STATE_NEW,
                                Mage_Sales_Model_Order::STATE_PENDING_PAYMENT
                            ))) {
                                $comment = Mage::helper('opay')->__('Payment canceled.');
                                $order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true, $comment, true);
                                $order->save();

                                Mage::getSingleton('checkout/session') -> getQuote() -> setIsActive(false) -> save();
                            }
                            exit('OK');

                        }
                        catch (Exception $e)
                        {
                            return get_class($e) . ': ' . $e->getMessage();
                        }
                    }
                }
                else
                {
                    return 'Order does not exist.';
                }
            }
            else
            {
                echo 'invalid signature';
            }
        }
    }

    public function validateUserRedirect()
    {
        require_once(Mage::getBaseDir() . '/app/code/local/Mage/Opay/OpayLibrary/opay_8.1.gateway.inc.php');
        $opay = new OpayGateway();
        $customerIds = Mage::getStoreConfig('payment/opay');

        if($customerIds['signature'] == 1)
        {
            $opay->setSignaturePassword($customerIds['api_secret']);
        }
        elseif ($customerIds['signature'] == 0)
        {
            $opay->setMerchantRsaPrivateKey($customerIds['api_rsa']);
            $opay->setOpayCertificate($customerIds['api_rsa_opay']);
        }

        if (isset($_POST['encoded']))
        {
            $parametersArray = $opay->convertEncodedStringToArrayOfParameters($_POST['encoded']);
        }
        else if (isset($_GET['encoded']))
        {
            $parametersArray = $opay->convertEncodedStringToArrayOfParameters($_GET['encoded']);
        }
        else if (isset($_POST['password_signature']) || isset($_POST['rsa_signature']))
        {
            $parametersArray = $_POST;
        }
        else if (isset($_GET['password_signature']) || isset($_GET['rsa_signature']))
        {
            $parametersArray = $_GET;
        }

        if (isset($parametersArray))
        {
            if ($opay->verifySignature($parametersArray))
            {
                $order = Mage::getModel('sales/order')->loadByIncrementId($parametersArray['order_nr']);
                if ($order->increment_id)
                {
                    if ($parametersArray['status'] == 1)
                    {

                        $receive_amount = intval($parametersArray['p_amount']);
                        $local_amount   = intval(number_format(($order->grand_total * 100), 0, '', ''));

                        if (($receive_amount != $local_amount) || ($parametersArray['currency'] != $order->order_currency_code))
                        {
                            $order->addStatusHistoryComment('Received payment doesn\'t match order\'s price! Received: ' . number_format($receive_amount, 2, ".", "") . ' ' . $parametersArray['currency']);
                            $order->save();
                            if (($receive_amount < $local_amount) || ($parametersArray['currency'] != $order->order_currency_code))
                            {
                                header('Location: '. Mage::getUrl('checkout/onepage/failure'));
                                exit();
                            }
                        }

                        try
                        {

                            if (in_array($order->getState(), array(
                                Mage_Sales_Model_Order::STATE_NEW,
                                Mage_Sales_Model_Order::STATE_PENDING_PAYMENT
                            ))) {

                                // Locking order until order state is changed to avoid possible double payment captures when user redirect and callback are happening at the exact same time
                                $connection = Mage::getSingleton('core/resource')->getConnection('core_read');
                                $connection->beginTransaction();
                                $query = 'SELECT * FROM ' . Mage::getSingleton('core/resource')->getTableName('sales/order') . ' WHERE increment_id = \'' . $parametersArray['order_nr'] . '\' FOR UPDATE';
                                $results = $connection->fetchAll($query);

                                // Capturing payment
                                if ($order->canInvoice()) {
                                    $invoice = $order->prepareInvoice();
                                    $invoice->register()->capture();
                                    $order->addRelatedObject($invoice);
                                }

                                // Sending e-mail notification to customer and adding comment to history about that
                                if ($invoice && !$order->getEmailSent()) {

                                    if (method_exists($order, 'queueNewOrderEmail')) {
                                        $order->queueNewOrderEmail();
                                    } else {
                                        $order->sendNewOrderEmail();
                                    }

                                    $comment = Mage::helper('opay')->__('Customer notified by E-mail.');
                                    $history = $order->addStatusHistoryComment($comment);
                                    $history->setIsCustomerNotified(true);
                                }

                                $order->save();
                                $connection->commit();

                                Mage::getSingleton('checkout/session') -> getQuote() -> setIsActive(false) -> save();
                            }

                            header('Location: '. Mage::getUrl('checkout/onepage/success'));
                            exit();
                        }
                        catch (Exception $e)
                        {
                            $connection->rollback();
                            return get_class($e) . ': ' . $e->getMessage();
                        }

                    }
                    elseif ($parametersArray['status'] == 2)
                    {
                        try
                        {
                            Mage::getSingleton('checkout/session') -> getQuote() -> setIsActive(false) -> save();
                            header('Location: '. Mage::getUrl('checkout/onepage/success'));
                            exit();
                        }
                        catch (Exception $e)
                        {
                            return get_class($e) . ': ' . $e->getMessage();
                        }

                    }
                }
                else
                {
                    return 'Order does not exist.';
                }
            }
            else
            {
                echo 'Invalid signature.';
            }
        }

        header('Location: '. Mage::getUrl('checkout/onepage/success'));
        exit();
    }

    private function appendMetadataToParams(&$paramsArray)
    {
        if (array_key_exists('metadata', $paramsArray)) {
            unset($paramsArray['metadata']);
        }

        try {
            $paramsArray['metadata']['app_version'] =  Mage::getVersion();
        } catch (Exception $exception) {
            // Do nothing to avoid any risks to break payment process
        }

        $paramsArray['metadata']['app_name'] = self::APP_NAME;
        $paramsArray['metadata']['widget_version'] = self::WIDGET_VERSION;
    }
}

